#!/bin/sh
#
# Written by Geoff Mottram (geoff at minaret dot biz).
#
# Placed in the public domain by the author.
#
# See LICENSE.txt for NO WARRANTY statement.
#

#
# This script will install a disk image to a target device.
# The image can be raw, or compressed with either
# "xz", "bzip2" or "gzip".
#
# The environment variable IMAGE_FILE must be set and should
# point to the image file to create.
#
# Usage:
#   IMAGE_FILE="../../images/disk.img.xz" sh disk_install.sh
#

#
# Configure the script.
#
config() {
    local NAME

    #
    # source functions
    #
    for NAME in "common.sh" "images.sh" "target.sh"; do
        NAME="functions/$NAME"
        if [ -f $NAME ]; then
            . "$NAME"
        else
            echo "\nCannot locate script file \"$NAME\". Exiting...\n"
            exit 1
        fi
    done

    # Check that IMAGE_FILE is set.
    image_file || quit

    # Check that IMAGE_FILE exists and is a regular file.
    image_exists || quit

    #
    # Validate image file name extension
    #
    case "$IMAGE_FILE" in
        *.img|*.xz|*.bzip2|*.gz) ;;
        *)  echo "\nInvalid file name extension on image file:\n\n$IMAGE_FILE\n"
            quit;;
    esac
}

#
# Display introduction screen.
#
intro() {
    clear
    cat <<END

This script will install a disk image to a target device.
The image can be raw, or compressed with either "xz",
"bzip2" or "gzip". This step can be skipped if you have
already loaded the image file onto the target device.

The image must have two partitions in which the root
partition is the second partition.

After the file has been copied, you will have the option to
expand the second partition on the target device to use
the entire space available on that device.

    IMAGE_FILE: $IMAGE_FILE
END
    prompt_enter
    clear
    cat <<END

The disk image will be copied to the target device with one
of the following commands, depending on the file name extension:

.img file:
    dd bs=1M if=IMAGE_FILE of=TARGET_DEVICE

.xz file:
    xzcat IMAGE_FILE | dd bs=1M of=TARGET_DEVICE

.bzip2 file:
    bzcat IMAGE_FILE | dd bs=1M of=TARGET_DEVICE

.gz file
    zcat IMAGE_FILE | dd bs=1M of=TARGET_DEVICE
END
    prompt_enter
    clear
    cat <<END

To expand the parition table and root parition to fill the
remainder of the device the following commands will be run:

    parted TARGET_DEVICE unit MiB print
    parted -a opt -- TARGET_DEVICE resizepart 2 -1
    e2fsck -f ROOT_DEVICE
    resize2fs ROOT_DEVICE

The first call to "parted" prints the partition table.
This should result in an offer to "Fix" the partition
table in order to  utilize the entire device.  You must
respond with "Fix". 

The second call to "parted" will expand the second
(root) partition to fill the remainder of the device.
END
    prompt_enter
}

#
# Main function, runs all steps.
#
main() {
    # Configure script
    config
    # Display intro screen
    intro
    # Run all three steps
    step1
    step2
    step3
    echo "\nInstallation complete.\n"
}

#
# Step 1
#
# Sets and exports TARGET_DEVICE, LOOP_DEVICE, ROOT_DEVICE and SECTORS.
#
step1() {
    select_target "--> Step 1: "
    if [ -z "$LOOP_DEVICE" ]; then
        ROOT_DEVICE="${TARGET_DEVICE}2"
    else
        ROOT_DEVICE="${TARGET_DEVICE}p2"
    fi
    export ROOT_DEVICE
}

#
# Step 2
#
# Installs the image file.
#
step2() {
    case "$IMAGE_FILE" in
        *.img) CMD="dd bs=1M if=\"$IMAGE_FILE\" of=\"$TARGET_DEVICE\"" ;;
        *.xz) CMD="xzcat \"$IMAGE_FILE\" | dd bs=1M of=\"$TARGET_DEVICE\"" ;;
        *.bzip2) CMD="bzcat \"$IMAGE_FILE\" | dd bs=1M of=\"$TARGET_DEVICE\"" ;;
        *.gz) CMD="zcat \"$IMAGE_FILE\" | dd bs=1M of=\"$TARGET_DEVICE\"" ;;
    esac
    clear
    cat <<END

--> Step 2: Install Image File on Target Device

TARGET_DEVICE: $TARGET_DEVICE

In this step the image file will be decompressed (if compressed) and
copied to the target device. This step can be skipped if you have
already done it.

The following command will be used:

    $CMD
END
    prompt_yn "Do you want to install the image file (y/n)? "
    if [ "$ANS" = "n" ]; then
        echo "\nSkipped installing image file."
        prompt_enter
        return
    fi
    # Need eval due to quotes and pipe
    if eval $CMD; then 
        echo "\nSuccessfully installed image file."
        prompt_enter
    else
        script_error $? "$CMD"
        quit
    fi
}

#
# Step 3
#
# Expand the partition table and root file system.
#
step3() {
    local PARTED
    clear
    cat <<END

--> Step 3: Expand Partition Table and Root File System

In this step you will expand the partition table to use
the remainder of the target device and resize the root
partition to use that space.

TARGET_DEVICE: $TARGET_DEVICE
  ROOT_DEVICE: $ROOT_DEVICE

Two calls to "parted" will be made:

    parted TARGET_DEVICE unit MiB print
    parted -a opt -- TARGET_DEVICE resizepart 2 -1

In the first call, "parted" should offer to "Fix" the
partition table to use the entire device. When prompted,
type in "Fix". The next call to "parted" will expand
partition 2 (root) to use the newly found space.

Then the root file system itself is extended.
END
    prompt_yn "Do you want to proceed (y/n)? "
    if [ "$ANS" = "n" ]; then
        echo "\nSkipped expanding partition table."
        prompt_enter
        return
    fi
    PARTED=$(which parted)
    if [ -z "$PARTED" ]; then
        echo "Please install parted (the GPT partition manager)"
        quit
    fi
    PRINT_CMD="parted $TARGET_DEVICE unit MiB print"
    CMD="$PRINT_CMD"
    echo "Expanding partition table with the following command:\n\n\t$CMD\n\nYou must input Fix, if asked.\n"
    if $CMD; then
        CMD="parted -a opt -- $TARGET_DEVICE resizepart 2 -1"
        cat <<END
Extending the second partition with the following command:

    $CMD

You may have to input the partition number (2) and
the max size (use the default offered by parted).

END
        if $CMD; then
            echo "Here is the final partition table:\n"
            $PRINT_CMD
        else
            script_error $? "$CMD"
            quit
        fi
    fi
    CMD="e2fsck -f $ROOT_DEVICE"
    echo "Performing root file system check with the following command:\n\n\t$CMD\n"
    if $CMD; then
        echo "\nRoot file system is clean"
    else
        script_error $? "$CMD"
        quit
    fi
    CMD="resize2fs $ROOT_DEVICE"
    echo "\nResizing root file system with the following command:\n\n\t$CMD\n"
    if $CMD; then
        echo "Successfully resized root file system"
    else
        script_error $? "$CMD"
        quit
    fi
}

#
# Run script
#
main

exit 0
