#!/bin/sh
#
# Written by Geoff Mottram (geoff at minaret dot biz).
#
# Placed in the public domain by the author.
#
# See LICENSE.txt for NO WARRANTY statement.
#

#
# Create a 2 GB sparse raw disk image (only 4K at end will be
# actually used).
#
# The environment variable IMAGE_FILE must be set and should
# point to the image file to create.
#
# Usage:
#   IMAGE_FILE="../../images/disk.img" sh disk_raw.sh [-b] [-s]
#
# Where:
#   -b (build) calls install.sh after device is created.
#   -s (script) option avoids almost all prompts
#
# The -b option requires the the environment variable BUILD_CMD
# to be set with the shell script to call after the loopback
# device has been created.
#
# The -b option causes the environment variable TARGET_DEVICE
# to be set to the "loopN" portion of the loopback device name
# before calling "$BUILD_CMD".
#
# 2 GB = 524288 4K blocks.
# LAST_BLOCK is one less than that.
#

# location of last block on new image
LAST_BLOCK="524287"

BUILD_OPTION=
SCRIPT_OPTION=

#
# Check command line arguments
#
for ARG in $@; do
    if [ "$ARG" = "-b" ]; then
        BUILD_OPTION="$ARG"
        if [ -z "$BUILD_CMD" ]; then
            echo "\n-b option requires BUILD_CMD environment variable to be set.\""
            exit 1;
        fi
    elif [ "$ARG" = "-s" ]; then
        SCRIPT_OPTION="$ARG"
    fi
done

#
# source functions
#
for NAME in "common.sh" "images.sh"; do
    NAME="functions/$NAME"
    if [ -f $NAME ]; then
        . "$NAME"
    else
        echo "\nCannot locate script file \"$NAME\". Exiting...\n"
        exit 1
    fi
done

# Check that the user is root
check_root

# Check that IMAGE_FILE is set.
image_file || exit 1

if [ -z "$SCRIPT_OPTION" ]; then
    clear
cat <<END

This script will create a 2 GiB raw disk image file and associate
it with a loopback device (i.e. "/dev/loop*"). If the image
file already exists, you will have an opportunity to recreate
it or use it as is.

The script will return the name of the loopback device that
is assigned by the system. You can use the "loop*" part
of the name in the Kirkwood installation script "install.sh".

In this way, you can create a raw disk image with all of the
partitions and file systems ready-to-go. There is a companion
script ("disk_pack.sh") which will disassociate the loopback
device from the raw image file and compress the resulting file.

Alternately, you can also use "unloop.sh" to disassociate the
loopback device from the raw image fie.
END
    prompt_enter
else
    echo "\n--> Creating 2GiB raw disk image in \"$IMAGE_FILE\"\n"
fi

# Make sure the IMAGE_FILE directory exists
DIR=$(dirname "$IMAGE_FILE")
if [ ! -e "$DIR" ]; then
    echo "\nThe image file directory \"$DIR\" does not exist, creating..."
    mkdir "$DIR" || exit 1
elif [ ! -d "$DIR" ]; then
    echo "\nThe image file directory \"$DIR\" is not a directory, exiting..."
    exit 1
fi

# disconnect raw image file from any loop devices
unloop || exit 1

if [ -e "$IMAGE_FILE" ]; then
    if [ ! -f "$IMAGE_FILE" ]; then
        echo "Existing raw image \"$IMAGE_FILE\" is not a regular file."
        quit
    fi
	if [ -z "$SCRIPT_OPTION" ]; then
        echo "Image file \"$IMAGE_FILE\" already exists."
        prompt_yn "Do you want to delete and recreate it (y/n)? "
    else 
        echo "Removing existing raw image \"$IMAGE_FILE\""
        ANS="y"
    fi
    if [ $ANS = "y" ]; then
        if rm "$IMAGE_FILE"; then
            echo "Removed existing raw image file \"$IMAGE_FILE\""
        else
            echo "Could not remove existing raw image file \"$IMAGE_FILE\"."
            quit
        fi
    fi
fi
            
if [ ! -e "$IMAGE_FILE" ]; then
    CMD="dd if=/dev/zero of=$IMAGE_FILE bs=4k seek=$LAST_BLOCK count=1"
    echo "Creating raw image file \"$IMAGE_FILE\" with the following command:\n\n\t$CMD\n"
    if $CMD; then
        :
    else
        echo "Could not create raw image file \"$IMAGE_FILE\"."
        quit
    fi
fi
CMD="losetup --show -P -f $IMAGE_FILE"
echo "\nCreating loop device for \"$IMAGE_FILE\" with the following command:\n\n\t$CMD\n"
if LOOP=$($CMD); then
    echo "Successfully created loopback device \"$LOOP\""
    if [ -n "$BUILD_OPTION" ]; then
        export TARGET_DEVICE=${LOOP#/dev/}
        CMD="$BUILD_CMD"
        echo "Calling installation script with the following command:\n\n\t$CMD\n"
        if $CMD; then
            :
        else
            script_error $? "$CMD"
            quit
        fi
    fi
else
    echo "Could not associate a loop device with raw image file \"$IMAGE_FILE\"."
    quit
fi

exit 0
