#!/bin/sh
#
# Written by Geoff Mottram (geoff at minaret dot biz).
#
# Placed in the public domain by the author.
#
# See LICENSE.txt for NO WARRANTY statement.
#

#
# Functions related to DTS files.
#

#
# Choose a DTS file.
#
# Sets DEVICE_TYPE (clears it if no selection is made).
#
dts_choose() {
    local ANS
    local LIST
    local PATTERN
    local FIRST
    local LAST
    local NAME
    local NUM

    # Reset DEVICE_TYPE
    DEVICE_TYPE=
    clear
    cat <<END

--> Choose a DTS file

The files in the boot/dts directory all start with "kirkwood-" and
end with ".dtb". These are binary versions of the source ".dts"
files. When prompted for a file name pattern here, don't include
the leading "kirkwood-".  Also, a trailing "*" will be added to
your search pattern for you. To list everything, just press ENTER.

Examples:
    lenovo
    *ix2
    openrd
    sheeva

Once you are in the list of DTS files, the following commands
are supported:

    n - next page
    p - previous page
    b - back to "search pattern" prompt
    r - return to Step 3: Select a DTS file
END
    while :; do
        echo -n "\nInput a search pattern: "
        read PATTERN
        PATTERN="kirkwood-$PATTERN*"
        LIST=$(cd ../dts && eval /bin/ls "$PATTERN" | sed -e 's/^kirkwood-//' -e 's/\.dtb$//')
        if [ -z "$LIST" ]; then
            continue
        fi
        set -- $LIST
        FIRST=1
        LAST=$#
        if [ $LAST -gt 20 ]; then
            LAST=20
        fi
        while :; do
            echo
            NUM=$FIRST
            while [ $NUM -le $LAST ]; do
                NAME="\${$NUM}"           # dash doesn't do $10 but ${10}
                eval echo "$NUM: $NAME"
                NUM=$(expr $NUM + 1)
            done
            while :; do
                echo -n "\nInput a number to select, blank or n=next page, p=prev page, b=back, r=return: "
                read ANS
                case $ANS in
                    ('' | n)
                        if [ $LAST -eq $# ]; then
                            echo "\nThis is the last page"
                        else
                            FIRST=$(expr $LAST + 1)
                            LAST=$(expr $FIRST + 19)
                            if [ $LAST -gt $# ]; then
                                LAST=$#
                            fi
                            break
                        fi
                        ;;
                    (p)
                        if [ $FIRST -eq 1 ]; then
                            echo "\nThis is the first page"
                        else
                            LAST=$(expr $FIRST - 1)
                            FIRST=$(expr $LAST - 19)
                            if [ "$FIRST" -lt 1 ]; then
                                FIRST=1
                            fi
                            break
                        fi
                        ;;
                    (b)
                        break 2     # return to pattern prompt
                        ;;
                    (r)
                        echo "\nNothing selected, returning..."
                        return
                        ;;
                    (*[!0-9]*)
                        echo "\nUknown command: \"$ANS\""
                        ;;
                    (*)
                        ANS=$(echo "$ANS" | sed 's/^0*//')  # remove leading zeros
                        if [ "$ANS" -ge 1 -a "$ANS" -le $# ]; then
                            NAME="\${$ANS}"
                            eval DEVICE_TYPE="$NAME"
                            return
                        else
                            echo "\nSelection #$ANS is out of range (must be from 1-$#)"
                        fi
                        ;;
                esac
            done
        done
    done
}

#
# Prompt user can input a DTS file name or choose one from a list.
#
dts_prompt() {
    cat << END

If you have a newer version of U-Boot which supports FTD kernels,
you don't have to select a DTS file. However, older U-Boot versions
(such as in the Lenovo IX2) require you to make a selection.
END
    prompt_yn "Do you need to select a DTS file [old U-Boot version] (y/n)? "
    if [ "$ANS" = "n" ]; then
        export DEVICE_TYPE="all"
        return
    fi
    cat << END

If you know the name of your DTS file (without the leading
"kirkwood-" and trailing ".dtb"), you can enter it directly.
Otherwise, you can pick from a list of all supported devices
in "boot/dts".

For IX2 users, the following DTS files are available:

    lenovo-ix2-dl
    lenovo-ix2-ng
END
    while :; do
        prompt_choice "Do you wan to input a DTS name (i) or choose from a list (c)? " i c
        if [ "$ANS" = "i" ]; then
            echo -n "\nInput a DTS name: "
            read DEVICE_TYPE
            if [ -z "$DEVICE_TYPE" ]; then
                echo "\nYou did not input anything."
            fi
        else
            dts_choose
        fi
        if [ -n "$DEVICE_TYPE" ]; then
            dts_validate
            if [ -n "$DEVICE_TYPE" ]; then
                prompt_yn "You selected \"$DEVICE_TYPE\", is this what you want (y/n)? "
                if [ "$ANS" = "y" ]; then
                    break
                fi
            fi
        fi
    done
    export DEVICE_TYPE
    #echo "\nSelected device type: $DEVICE_TYPE"
    #prompt_enter
}

#
# Check that a DTS name exists in the dts directory.
#
# Checks $DEVICE_TYPE, clearing it if invalid.
#
dts_validate() {
    local TYPE=${DEVICE_TYPE#kirkwood-}
    DEVICE_TYPE=${TYPE%.dtb}
    if [ "X$DEVICE_TYPE" = "Xall" ]; then
        return
    fi
    if [ ! -e "../dts/kirkwood-$TYPE.dtb" ]; then
        echo "\nInvalid DEVICE_TYPE: \"$DEVICE_TYPE\""
        DEVICE_TYPE=""
    fi
}
