#!/bin/sh
#
# Written by Geoff Mottram (geoff at minaret dot biz).
#
# Placed in the public domain by the author.
#
# See LICENSE.txt for NO WARRANTY statement.
#

#
# This script installs the kernel and initial RAM disk on a target device.
# This should only be run for devices that have an older version
# of U-Boot that does not support GUID partition tables. It installs
# the kernel and initial RAM disk in an unallocated portion of the
# disk before the first partition.
#
# Usage:
#
# sh install_images.sh [-s]
#
# Where:
#   -s (script) option avoids almost all prompts
#


#
# Global variables
#
# Non-zero if "-s" (script) option was passed on the command line
SCRIPT_OPTION=

# Name of the kernel file
FILE_KERNEL=uImage

# Name of the initial RAM disk file
FILE_INITRD=uInitrd

# Hard drive to copy boot files to
# This must be set outside of this script.
# Either with leading "/dev/" or without.
#TARGET_DEVICE=sdX
#TARGET_DEVICE=loopN
#TARGET_DEVICE=/dev/sdX
#TARGET_DEVICE=/dev/loopN

# Number of sectors to seek before copying kernel
SEEK_KERNEL=2048

# Number of sectors to seek before copying initial RAM disk
SEEK_INITRD=34816

# Maximum kernel file size in sectors
# 16386 sectors = 8 MiB
MAX_KERNEL=16384

# Maximum initial RAM disk file size in sectors
# 24576 sectors = 12 MiB
MAX_INITRD=24576

#
# Check command line arguments
#
for ARG in $@; do
    if [ "$ARG" = "-s" ]; then
        SCRIPT_OPTION="$ARG"
    fi
done

# check the current directory
if [ ! -d ../dts ]; then
    cat <<END

You appear to be in the wrong directory.
Cannot locate "../dts" directory.
You must run this script from the "boot/tools" directory.

END
    exit 1
fi

#
# source functions
#
for NAME in "common.sh" "target.sh"; do
    NAME="functions/$NAME"
    if [ -f $NAME ]; then
        . "$NAME"
    else
        echo "\nCannot locate script file \"$NAME\". Exiting...\n"
        exit 1
    fi
done

#
# Copy kernel or initial RAM disk using dd.
#
# Usage: copy_file FILE_TYPE FILE_NAME SEEK_TO
#
copy_file() {
    local CMD="dd if=$2 of=$TARGET_DEVICE bs=1b seek=$3"
    echo "\nCopying $1 using: $CMD"
    if $CMD; then
        echo "Successsfully copied $1"
    else
        script_error $? "$CMD"
        quit
    fi
}

#
# Check the size of a file in sectors.
#
# Usage: size_check FILE_TYPE FILE_NAME MAX_SECTORS 
#
# Result is placed in $SECTORS
#
size_check() {
    local FILE_TYPE=$1
    local FILE_NAME=$2
    local MAX_SECTORS=$3
    if [ -f $FILE_NAME ]; then
        set `wc -c $FILE_NAME`
        SECTORS=`expr $1 / 512`
        local MOD=`expr $1 % 512`
        if [ $MOD -ne 0 ]; then
            SECTORS=`expr $SECTORS + 1`
        fi
        echo "\nThe $FILE_TYPE file \"$FILE_NAME\" is $SECTORS sectors."
        if [ $SECTORS -gt $MAX_SECTORS ]; then
            echo "This is larger than the current maximum size of $MAX_SECTORS sectors."
            echo "You will need to change the U-Boot environment variables which provide"
            echo "the kernel and initrd offsets and sizes and then change this script."
            exit 1
        fi
        echo "This is within the current maximum size of $MAX_SECTORS sectors."
    else
        echo "\nCannot find $FILE_TYPE file \"$FILE_NAME\""
        exit 1
    fi
}

# Validate $TARGET_DEVICE and prompt user if not set or invalid
check_target

# Check that the user is root
check_root

if [ -z "$SCRIPT_OPTION" ]; then
    clear
    cat <<END

--> Install Kernel and Initial RAM Disk

This script will install the kernel from "uImage" and the
initial RAM disk from "uInitrd" to the reserved space before
the first partition on the device:

    $TARGET_DEVICE

You can choose to install one, both or neither of these files
(in which case, the "dd" commands are just printed out).

*** WARNING ***
This script copies data to a 32 MB area of the hard disk that
is being used to store the Linux kernel and initial RAM disk.
This space is located after the partition table and before
the first partition (swap). You must run the "install.sh"
installation script or manually partition the disk in a
similar way to insure that this space has been set aside.

*** IMPORTANT ***
Do not run this script if you have a newer version of U-Boot
that supports GUID Partition Tables (GPT) and Flattened Device
Tree (FDT) kernels. You should be able to boot directly from
the "/boot" directory of the root file system.
END
    prompt_y_or_quit
    clear
else
    echo "\n--> Install Kernel and Initial RAM Disk on device \"$TARGET_DEVICE\""
fi

# Change to parent directory
cd ..

#
# Fetch the sizes of the kernel and initial RAM disk files in sectors
#
size_check "kernel" $FILE_KERNEL $MAX_KERNEL
SIZE_KERNEL=$SECTORS
size_check "initial RAM disk" $FILE_INITRD $MAX_INITRD
SIZE_INITRD=$SECTORS

if [ -z "$SCRIPT_OPTION" ]; then
    #
    # Display the partition table using parted
    #
    PARTED=`which parted`
    if [ -z "$PARTED" ]; then
        echo "Please install parted (the GPT partition manager)"
        quit
    fi

    CMD="parted $TARGET_DEVICE unit MiB print"
    echo
    echo "Displaying partition table by running \"$CMD\""
    echo
    echo "*****PARTITION TABLE ($TARGET_DEVICE)****"
    echo
    if $CMD; then
        echo "*****END OF PARTITION TABLE****"
    else
        script_error $? "$CMD"
        quit
    fi
    echo "\nMake sure that the first partition does not start before 33 MiB."
    prompt_yn "Do you want to install the kernel file \"$FILE_KERNEL\" (y/n)? "
else
    ANS="y"
fi

if [ $ANS = "y" ]; then
    copy_file kernel $FILE_KERNEL $SEEK_KERNEL
else
    echo
    echo "Kernel file not installed. The command would have been:"
    echo
    echo "    dd if=$FILE_KERNEL of=$TARGET_DEVICE bs=1b seek=$SEEK_KERNEL"
fi

if [ -z "$SCRIPT_OPTION" ]; then
    prompt_yn "Do you want to install the initial RAM disk file \"$FILE_INITRD\" (y/n)? "
else
    ANS="y"
fi
if [ $ANS = "y" ]; then
    copy_file "initial RAM disk" $FILE_INITRD $SEEK_INITRD
else
    echo
    echo "Initial RAM disk not installed. The command would have been:"
    echo
    echo "    dd if=$FILE_INITRD of=$TARGET_DEVICE bs=1b seek=$SEEK_INITRD"
fi
