#!/bin/sh
#
# Written by Geoff Mottram (geoff at minaret dot biz).
#
# Placed in the public domain by the author.
#
# See LICENSE.txt for NO WARRANTY statement.
#

#
# This script generates the kernel and initial RAM disk.
#
# Usage:
#
# From the /boot directory:
#
# sh make_images.sh [-s]
#
# Where:
#   -s (script) option avoids almost all prompts
#

#
# If the DEVICE_TYPE environment variable is not set,
# you will be prompted for a DTS device type.
#
# A DEVICE_TYPE of "all" will generate a kernel that requires a newer
# U-Boot version that supports Flattened Device Tree (FDT) kernels.
#

SCRIPT_OPTION=

#
# Check command line arguments
#
for ARG in $@; do
    if [ "$ARG" = "-s" ]; then
        SCRIPT_OPTION="$ARG"
    fi
done

# check the current directory
if [ ! -d ../dts ]; then
    cat <<END

You appear to be in the wrong directory.
Cannot locate "../dts" directory.
You must run this script from the "boot/tools" directory.

END
    exit 1
fi

#
# source functions
#
for NAME in "common.sh" "dts.sh" "kernel_version.sh"; do
    NAME="functions/$NAME"
    if [ -f $NAME ]; then
        . "$NAME"
    else
        echo "\nCannot locate script file \"$NAME\". Exiting...\n"
        exit 1
    fi
done

# Check that the user is root
check_root

if [ -z "$KERNEL_VERSION" ]; then
    echo "\nKERNEL_VERSION is not set.\nIt should be found in kernel_version.sh\n"
    exit 1
fi

#
# Kernel and initial ram disk files
#
KERNEL_NAME="Linux-${KERNEL_VERSION}-kirkwood-tld-1"
KERNEL_IMAGE="vmlinuz-${KERNEL_VERSION}-kirkwood-tld-1"
INITRD_NAME="initramfs-${KERNEL_VERSION}-kirkwood-tld-1"
INITRD_IMAGE="initrd.img-${KERNEL_VERSION}-kirkwood-tld-1"

#
# Check files exist
#
NAME="../$KERNEL_IMAGE"
if [ ! -f "$NAME" ]; then
    echo "\nKernel image \"$NAME\" does not exist or is not a regular file"
    quit
fi
NAME="../$INITRD_IMAGE"
if [ ! -f "$NAME" ]; then
    echo "\nInitrd image \"$NAME\" does not exist or is not a regular file"
    quit
fi

#
# If DEVICE_TYPE is not set or is invalid,
# display instructions and prompt for a DTS.
#
if [ -n "$DEVICE_TYPE" ]; then
    dts_validate
fi
if [ -z "$SCRIPT_OPTION" ]; then
    clear
    cat <<END

--> Generate Kernel and Initial RAM Disk

This script generates the kernel (uImage) and initial RAM disk (uInitrd)
with the following settings:

    KERNEL VERSION: $KERNEL_VERSION
       KERNEL FILE: $KERNEL_IMAGE
       INITRD FILE: $INITRD_IMAGE

To change the version number, edit "boot/tools/functions/kernel_version.sh"
and run this script again.
END
else
    echo "\n--> Generate Kernel and Initial RAM Disk\n"
fi
if [ -z "$DEVICE_TYPE" ]; then
    dts_prompt
elif [ -z "$SCRIPT_OPTION" ]; then
    prompt_enter
fi

if [ "$DEVICE_TYPE" != "all" ]; then
    DTS="kirkwood-${DEVICE_TYPE}.dtb"
    echo "\nGenerate a kernel for older U-Boot versions."
    echo "\nUsing DTS file: $DTS"
else
    echo "\nGenerate a kernel for newer U-Boot versions which support"
    echo "Flattened Device Tree (FDT) booting."
fi

# Change to parent directory

cd ..

echo "\nCreate kernel (uImage)"

if [ "$DEVICE_TYPE" != "all" ]; then
    echo "\nCopy $KERNEL_IMAGE to zImage.fdt and append dts/$DTS"
    cp -a "$KERNEL_IMAGE" zImage.fdt
    cat "dts/$DTS" >> zImage.fdt
    KERNEL_IMAGE="zImage.fdt"
fi

CMD="mkimage -A arm -O linux -T kernel -C none -a 0x00008000 -e 0x00008000 -n $KERNEL_NAME -d $KERNEL_IMAGE uImage"
echo "\nRunning mkimage as follows:\n\n$CMD\n"

if $CMD; then
    echo "\nSuccessfully created kernel (uImage)"
    # clean up
    rm -f zImage.fdt
else
    script_error $? "$CMD"
    quit
fi

echo "\nCreate initial RAM disk (uInitrd)"
CMD="mkimage -A arm -O linux -T ramdisk -C gzip -a 0x00000000 -e 0x00000000 -n $INITRD_NAME -d $INITRD_IMAGE uInitrd"
echo "\nRunning mkimage as follows:\n\n$CMD\n"

if $CMD; then
    echo "\nSuccessfully created initial RAM disk (uInitrd)"
else
    script_error $? "$CMD"
    quit
fi

exit 0
